@@ -17,97 +17,35 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 
-import threading, time, Queue, os, sys, shutil
-from util import user_dir, appdata_dir, print_error
+import os
+import util
 from bitcoin import *
 
 
-class Blockchain(threading.Thread):
-
+class Blockchain():
+    '''Manages blockchain headers and their verification'''
     def __init__(self, config, network):
-        threading.Thread.__init__(self)
-        self.daemon = True
         self.config = config
         self.network = network
-        self.lock = threading.Lock()
-        self.local_height = 0
-        self.running = False
         self.headers_url = 'http://headers.electrum.org/blockchain_headers'
+        self.local_height = 0
         self.set_local_height()
-        self.queue = Queue.Queue()
 
-    
+    def print_error(self, *msg):
+        util.print_error("[blockchain]", *msg)
+
     def height(self):
         return self.local_height
 
-
-    def stop(self):
-        with self.lock: self.running = False
-
-
-    def is_running(self):
-        with self.lock: return self.running
-
-
-    def run(self):
+    def init(self):
         self.init_headers_file()
         self.set_local_height()
-        print_error( "blocks:", self.local_height )
+        self.print_error("%d blocks" % self.local_height)
 
-        with self.lock:
-            self.running = True
-
-        while self.is_running():
-
-            try:
-                result = self.queue.get()
-            except Queue.Empty:
-                continue
-
-            if not result: continue
-
-            i, header = result
-            if not header: continue
-            
-            height = header.get('block_height')
-
-            if height <= self.local_height:
-                continue
-
-            if height > self.local_height + 50:
-                if not self.get_and_verify_chunks(i, header, height):
-                    continue
-
-            if height > self.local_height:
-                # get missing parts from interface (until it connects to my chain)
-                chain = self.get_chain( i, header )
-
-                # skip that server if the result is not consistent
-                if not chain: 
-                    print_error('e')
-                    continue
-                
-                # verify the chain
-                if self.verify_chain( chain ):
-                    print_error("height:", height, i.server)
-                    for header in chain:
-                        self.save_header(header)
-                else:
-                    print_error("error", i.server)
-                    # todo: dismiss that server
-                    continue
-
-
-            self.network.new_blockchain_height(height, i)
-
-
-                    
-            
     def verify_chain(self, chain):
-
         first_header = chain[0]
         prev_header = self.read_header(first_header.get('block_height') -1)
-        
+
         for header in chain:
 
             height = header.get('block_height')
@@ -118,7 +56,7 @@ class Blockchain(threading.Thread):
             try:
                 assert prev_hash == header.get('prev_block_hash')
                 assert bits == header.get('bits')
-                assert eval('0x'+_hash) < target
+                assert int('0x'+_hash,16) < target
             except Exception:
                 return False
 
@@ -133,7 +71,7 @@ class Blockchain(threading.Thread):
         height = index*2016
         num = len(data)/80
 
-        if index == 0:  
+        if index == 0:
             previous_hash = ("0"*64)
         else:
             prev_header = self.read_header(index*2016-1)
@@ -149,45 +87,15 @@ class Blockchain(threading.Thread):
             _hash = self.hash_header(header)
             assert previous_hash == header.get('prev_block_hash')
             assert bits == header.get('bits')
-            assert eval('0x'+_hash) < target
+            assert int('0x'+_hash,16) < target
 
             previous_header = header
-            previous_hash = _hash 
+            previous_hash = _hash
 
         self.save_chunk(index, data)
-        print_error("validated chunk %d"%height)
+        self.print_error("validated chunk %d to height %d" % (index, height))
 
 
-    def verify_header(self, header):
-        # add header to the blockchain file
-        # if there is a reorg, push it in a stack
-
-        height = header.get('block_height')
-
-        prev_header = self.read_header(height -1)
-        if not prev_header:
-            # return False to request previous header
-            return False
-
-        prev_hash = self.hash_header(prev_header)
-        bits, target = self.get_target(height/2016)
-        _hash = self.hash_header(header)
-        try:
-            assert prev_hash == header.get('prev_block_hash')
-            assert bits == header.get('bits')
-            assert eval('0x'+_hash) < target
-        except Exception:
-            # this can be caused by a reorg.
-            print_error("verify header failed"+ repr(header))
-            verifier.undo_verifications()
-
-            # return False to request previous header.
-            return False
-
-        self.save_header(header)
-        print_error("verify header:", _hash, height)
-        return True
-        
 
     def header_to_string(self, res):
         s = int_to_hex(res.get('version'),4) \
@@ -200,7 +108,7 @@ class Blockchain(threading.Thread):
 
 
     def header_from_string(self, s):
-        hex_to_int = lambda s: eval('0x' + s[::-1].encode('hex'))
+        hex_to_int = lambda s: int('0x' + s[::-1].encode('hex'), 16)
         h = {}
         h['version'] = hex_to_int(s[0:4])
         h['prev_block_hash'] = hash_encode(s[4:36])
@@ -214,21 +122,20 @@ class Blockchain(threading.Thread):
         return rev_hex(Hash(self.header_to_string(header).decode('hex')).encode('hex'))
 
     def path(self):
-        return os.path.join( self.config.path, 'blockchain_headers')
+        return os.path.join(self.config.path, 'blockchain_headers')
 
     def init_headers_file(self):
         filename = self.path()
         if os.path.exists(filename):
             return
-        
         try:
             import urllib, socket
             socket.setdefaulttimeout(30)
-            print_error("downloading ", self.headers_url )
+            self.print_error("downloading ", self.headers_url )
             urllib.urlretrieve(self.headers_url, filename)
-            print_error("done.")
+            self.print_error("done.")
         except Exception:
-            print_error( "download failed. creating file", filename )
+            self.print_error( "download failed. creating file", filename )
             open(filename,'wb+').close()
 
     def save_chunk(self, index, chunk):
@@ -250,7 +157,6 @@ class Blockchain(threading.Thread):
         f.close()
         self.set_local_height()
 
-
     def set_local_height(self):
         name = self.path()
         if os.path.exists(name):
@@ -258,7 +164,6 @@ class Blockchain(threading.Thread):
             if self.local_height != h:
                 self.local_height = h
 
-
     def read_header(self, block_height):
         name = self.path()
         if os.path.exists(name):
@@ -268,10 +173,11 @@ class Blockchain(threading.Thread):
             f.close()
             if len(h) == 80:
                 h = self.header_from_string(h)
-                return h 
+                return h
 
-
-    def get_target(self, index, chain=[]):
+    def get_target(self, index, chain=None):
+        if chain is None:
+            chain = []  # Do not use mutables as default values!
 
         max_target = 0x00000000FFFF0000000000000000000000000000000000000000000000000000
         if index == 0: return 0x1d00ffff, max_target
@@ -282,13 +188,13 @@ class Blockchain(threading.Thread):
             for h in chain:
                 if h.get('block_height') == index*2016-1:
                     last = h
- 
+
         nActualTimespan = last.get('timestamp') - first.get('timestamp')
         nTargetTimespan = 14*24*60*60
         nActualTimespan = max(nActualTimespan, nTargetTimespan/4)
         nActualTimespan = min(nActualTimespan, nTargetTimespan*4)
 
-        bits = last.get('bits') 
+        bits = last.get('bits')
         # convert to bignum
         MM = 256*256*256
         a = bits%MM
@@ -298,7 +204,7 @@ class Blockchain(threading.Thread):
 
         # new target
         new_target = min( max_target, (target * nActualTimespan)/nTargetTimespan )
-        
+
         # convert it to bits
         c = ("%064X"%new_target)[2:]
         i = 31
@@ -306,121 +212,46 @@ class Blockchain(threading.Thread):
             c = c[2:]
             i -= 1
 
-        c = eval('0x'+c[0:6])
-        if c > 0x800000: 
+        c = int('0x'+c[0:6],16)
+        if c >= 0x800000:
             c /= 256
             i += 1
 
         new_bits = c + MM * i
         return new_bits, new_target
 
+    def connect_header(self, chain, header):
+        '''Builds a header chain until it connects.  Returns True if it has
+        successfully connected, False if verification failed, otherwise the
+        height of the next header needed.'''
+        chain.append(header)  # Ordered by decreasing height
+        previous_height = header['block_height'] - 1
+        previous_header = self.read_header(previous_height)
 
-    def request_header(self, i, h, queue):
-        print_error("requesting header %d from %s"%(h, i.server))
-        i.send([ ('blockchain.block.get_header',[h])], lambda i,r: queue.put((i,r)))
-
-    def retrieve_header(self, i, queue):
-        while True:
-            try:
-                ir = queue.get(timeout=1)
-            except Queue.Empty:
-                print_error('timeout')
-                continue
-
-            if not ir: 
-                continue
-
-            i, r = ir
-
-            if r.get('error'):
-                print_error('Verifier received an error:', r)
-                continue
-
-            # 3. handle response
-            method = r['method']
-            params = r['params']
-            result = r['result']
-
-            if method == 'blockchain.block.get_header':
-                return result
-                
-
-
-    def get_chain(self, interface, final_header):
-
-        header = final_header
-        chain = [ final_header ]
-        requested_header = False
-        queue = Queue.Queue()
-
-        while self.is_running():
-
-            if requested_header:
-                header = self.retrieve_header(interface, queue)
-                if not header: return
-                chain = [ header ] + chain
-                requested_header = False
-
-            height = header.get('block_height')
-            previous_header = self.read_header(height -1)
-            if not previous_header:
-                self.request_header(interface, height - 1, queue)
-                requested_header = True
-                continue
-
-            # verify that it connects to my chain
-            prev_hash = self.hash_header(previous_header)
-            if prev_hash != header.get('prev_block_hash'):
-                print_error("reorg")
-                self.request_header(interface, height - 1, queue)
-                requested_header = True
-                continue
-
-            else:
-                # the chain is complete
-                return chain
-
-
-    def get_and_verify_chunks(self, i, header, height):
-        requested_chunks = []
-        queue = Queue.Queue()
-        min_index = (self.local_height + 1)/2016
-        max_index = (height + 1)/2016
-
-        for n in range(min_index, max_index + 1):
-            i.send([ ('blockchain.block.get_chunk',[n])], lambda i,r:queue.put(r))
-            requested_chunks.append(n)
-
-        print_error( "requested chunks:", requested_chunks )
-
-        while requested_chunks:
-            try:
-                r = queue.get(timeout=1)
-            except Queue.Empty:
-                continue
-            if not r: continue
-
-            if r.get('error'):
-                print_error('Verifier received an error:', r)
-                continue
-
-            # 3. handle response
-            params = r['params']
-            result = r['result']
-
-            index = params[0]
-            try:
-                self.verify_chunk(index, result)
-            except Exception:
-                print_error('Verify chunk failed!!')
-                return False
-            requested_chunks.remove(index)
-
-        return True
-
-
-
+        # Missing header, request it
+        if not previous_header:
+            return previous_height
 
+        # Does it connect to my chain?
+        prev_hash = self.hash_header(previous_header)
+        if prev_hash != header.get('prev_block_hash'):
+            self.print_error("reorg")
+            return previous_height
 
+        # The chain is complete.  Reverse to order by increasing height
+        chain.reverse()
+        if self.verify_chain(chain):
+            self.print_error("connected at height:", previous_height)
+            for header in chain:
+                self.save_header(header)
+            return True
 
+        return False
 
+    def connect_chunk(self, idx, chunk):
+        try:
+            self.verify_chunk(idx, chunk)
+            return idx + 1
+        except Exception:
+            self.print_error('verify_chunk failed')
+            return idx - 1
